@@ -1,6 +1,6 @@
 ## RcppEigen.package.skeleton.R: makes a skeleton for a package that wants to use RcppEigen
 ##
-## Copyright (C) 2011 - 2014  Douglas Bates, Dirk Eddelbuettel and Romain Francois
+## Copyright (C) 2011 - 2015  Douglas Bates, Dirk Eddelbuettel and Romain Francois
 ##
 ## This file is part of RcppEigen.
 ##
@@ -31,21 +31,27 @@ RcppEigen.package.skeleton <- function(name= "anRpackage", list = character(),
     } else {
         fake <- FALSE
     }
-	
+
+    haveKitten <- requireNamespace("pkgKitten", quietly=TRUE, character.only=TRUE)
+    skelFunUsed <- ifelse(haveKitten, "kitten", "package.skeleton")
+    message("\nCalling ", skelFunUsed, " to create basic package.")
+
     ## first let the traditional version do its business
     call <- match.call()
-    call[[1]] <- as.name("package.skeleton")
-    if ("example_code" %in% names(call)) {
-        ## remove the example_code argument
-        call[["example_code"]] <- NULL
-    }
-    if (fake) {
-        call[["list"]] <- "Rcpp.fake.fun"
+    call[[1]] <- if (haveKitten) as.name("kitten") else as.name("package.skeleton")
+    if (! haveKitten) {                 # in the package.skeleton() case
+        if ("example_code" %in% names(call)) {
+            call[["example_code"]] <- NULL    # remove the example_code argument
+        }
+        if (fake) {
+            call[["list"]] <- "Rcpp.fake.fun"
+        }
     }
 	
-    tryCatch(eval(call, envir = env), error = function(e) {
-        stop("error while calling `package.skeleton`")
-    })
+    tryCatch(eval(call, envir = env),
+             error = function(e) {
+                 stop(paste("error while calling `", skelFunUsed, "`", sep=""))
+             })
 	
     message("\nAdding RcppEigen settings")
 	
@@ -65,7 +71,7 @@ RcppEigen.package.skeleton <- function(name= "anRpackage", list = character(),
         message(" >> added LinkingTo: Rcpp, RcppEigen")
     }
 	
-    ## if there is a NAMESPACE, add a useDynLib
+    ## add a useDynLib to NAMESPACE, 
     NAMESPACE <- file.path(root, "NAMESPACE")
     lines <- readLines(NAMESPACE)
     if (! grepl("useDynLib", lines)) {
@@ -74,7 +80,7 @@ RcppEigen.package.skeleton <- function(name= "anRpackage", list = character(),
                    "importFrom(Rcpp, evalCpp)",        ## ensures Rcpp instantiation
                    lines)
         writeLines(lines, con = NAMESPACE)
-        message(" >> added useDynLib directive to NAMESPACE")
+        message(" >> added useDynLib and importFrom directives to NAMESPACE")
     }
 	
     ## lay things out in the src directory
@@ -82,22 +88,28 @@ RcppEigen.package.skeleton <- function(name= "anRpackage", list = character(),
     if (!file.exists(src)) {
         dir.create(src)
     }
+    man <- file.path(root, "man")
+    if (!file.exists(man)) {
+        dir.create(man)
+    }
     skeleton <- system.file("skeleton", package = "RcppEigen")
     Makevars <- file.path(src, "Makevars")
     if (!file.exists(Makevars)) {
         file.copy(file.path(skeleton, "Makevars"), Makevars)
-        message(" >> added Makevars file")
+        message(" >> added Makevars file with RcppEigen settings")
     }
 	
     Makevars.win <- file.path(src, "Makevars.win")
     if (!file.exists(Makevars.win)) {
         file.copy(file.path(skeleton, "Makevars.win"), Makevars.win)
-        message(" >> added Makevars.win file")
+        message(" >> added Makevars.win file with RcppEigen settings")
     }
 		
     if (example_code) {
         file.copy(file.path(skeleton, "rcppeigen_hello_world.cpp"), src)
         message(" >> added example src file using Eigen classes")
+        file.copy(file.path(skeleton, "rcppeigen_hello_world.Rd"), man)
+        message(" >> added example Rd file for using Eigen classes")
 
 	Rcpp::compileAttributes(root)
         message(" >> invoked Rcpp::compileAttributes to create wrappers")
@@ -107,7 +119,6 @@ RcppEigen.package.skeleton <- function(name= "anRpackage", list = character(),
         rm("Rcpp.fake.fun", envir = env)
         unlink(file.path(root, "R"  , "Rcpp.fake.fun.R"))
         unlink(file.path(root, "man", "Rcpp.fake.fun.Rd"))
-        
     }
 	
     invisible(NULL)
